﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class IfThenElseStat : CodeNode {
    public override int MinimumArity { get { return 2; } }
    public override int MaximumArity { get { return 3; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    protected IfThenElseStat(bool deserializing) : base(deserializing) { }
    protected IfThenElseStat(IfThenElseStat original, Cloner cloner) : base(original, cloner) { }

    public IfThenElseStat() : base("IfThenElseStat", "An if statement.") { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IfThenElseStat(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, IEnumerable<ISymbolicExpressionTreeNode> children) {
      ISymbolicExpressionTreeNode condition = null, truePart = null, falsePart = null;
      string[] parts = new string[3];

      if (children.Count() < 2 || children.Count() > 3)
        throw new Exception("Unexpected number of children. Expected 2 or 3 children.");

      var childLst = children.ToList();
      condition = childLst[0];
      truePart = childLst[1];
      if (childLst.Count == 3)
        falsePart = childLst[2];

      parts[0] = ((CodeNode)condition.Symbol).Interpret(condition, condition.Subtrees);
      parts[1] = ((CodeNode)truePart.Symbol).Interpret(truePart, truePart.Subtrees);
      if (childLst.Count == 3) parts[2] = ((CodeNode)falsePart.Symbol).Interpret(falsePart, falsePart.Subtrees);

      string result = "if (" + parts[0] + ") {" + parts[1] + "}";
      if (!string.IsNullOrEmpty(parts[2]))
        result += " else {" + result[2] + "}";
      return result;
    }
  }
}
